# frozen_string_literal: true

module Hermes
  module Api
    module Dmo
      class HousingChannelSettingsController < Hermes::Api::Dmo::ApplicationController
        hermes_errors 'housing_channel_error'

        before_action :authorize_administrator!
        set_instance_for %i[index show create update destroy]

        rescue_from Hermes::Errors::HousingChannelError do |exception|
          render_json_error(:unprocessable_entity, :housing_channel_error, { message: exception.message })
        end

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $eq
          $all
        ]

        ACCEPTABLE_PARAMS = %w[
          department
          insee_code
          housing_channel_keys
          created_at
          updated_at
        ]

        MUTATE_PARAMS = %i[
          department
          insee_code
          housing_channel_keys
        ]
        
        def mutate_params
          MUTATE_PARAMS
        end 


        def index
          acceptable_params *ACCEPTABLE_PARAMS
          valid_operators *VALID_OPERATORS
          super
        end

        def authorize_administrator!
          return render_json_error(:unauthorized, :not_admin) unless current_user_roles.include?('Administrator')
        end
      end
    end
  end
end