# frozen_string_literal: true

module Hermes
  module Api
    module Dmo
      # checker Api::Dmo::HermesController avant d'ajouter des methodes
      class HousingOffersController < Hermes::Api::Dmo::ApplicationController
              hermes_errors 'housing_channel_error'

        require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s
        set_instance_for :eligible_requests, :with_validated_applications_for, :history

        VALIDATED_APPLICATION_STATUS = 'TRANSMIS BAILLEUR'

        MANAGEMENT_ROLES = [
          'Administrator',
          'Manager',
          'superadministrateur',
          'entreprise',
          'bailleur',
          'collaborateur offre logement'
        ].freeze

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          lessor_code
          is_from_form
          requalification
          dalo
          ids
          als_id
          entry_date
          created_at
          updated_at
          external_ref
          communicate_exact_address
          eligibility_query
          typology
          lessor_status
          type
          age
          district
          department
          publication_end_date
          expiry_days
          rent_amount
          rent_with_charges
          surface
          rooms
          exclusivity_end_date
          nb_of_eligible_employees
          nb_of_requests_from_employees
          availability_date
          description
          details
          old_status
          status_updated_at
          effort_rate
          min_effort_treshold
          single_parent
          reserved
          priority_public
          mobility
          application_following_contact
          bedrooms
          bathrooms
          heating_type
          hot_water_distribution
          floors
          elevators
          has_elevator
          has_gardian
          outside
          cellars
          has_cellar
          parking_charges
          parking_type
          pmr
          kitchen_type
          mutation
          threshold_category_1
          threshold_type
          threshold_subtype
          threshold_category_2
          threshold_category_3
          threshold_category_4
          threshold_category_5
          threshold_category_6
          threshold_category_7
          threshold_category_8
          threshold_category_9
          threshold_category_10
          threshold_category_11
          threshold_category_12
          individual_heating
          date_publication_start
          date_publication_end
          lessor_id
          kind
          building_type
          already_rent
          guarantee_deposit
          rental_charges
          offer_status
          offer_status_updated_at
          lessor_status_updated_at
          ceil_category_1
          ceil_category_2
          ceil_category_3
          ceil_category_4
          ceil_category_5
          ceil_category_6
          ceil_category_7
          ceil_category_8
          ceil_category_9
          ceil_category_10
          ceil_category_11
          ceil_category_12
          floor_category_1
          floor_category_2
          floor_category_3
          floor_category_4
          floor_category_5
          floor_category_6
          floor_category_7
          floor_category_8
          floor_category_9
          floor_category_10
          floor_category_11
          floor_category_12
          min_revenues
          total_charges
          max_effort_rate
          charges
          tense_area
          subsidiary_company
          rpls_id
          operation_name
          administrator
          administration_area
          potential_request_name_town
          potential_request_name_county
          number_of_companies_30_minutes_around
          construction_type
          construction_year
          address_can_be_shared
          qpv
          total_floors
          address_extended
          test_created_at
          already_inhabited
          new_offer
          lessor_name
          lessor_siren
          als_siren
          postal_code
          insee_code
          postal_codes
          residence_code
          residence_title
          pictures
          agency_id
          agency_name
          lessor_contact_name
          lessor_email
          lessor_phone_number
          gardens
          has_garden
          green_area
          condo
          furnished_lodging
          virtual_visit
          duplex
          first_quartile
          other_quartiles
          lessor_convention_reference
          disability_accesses
          has_disability_access
          terraces_or_balconies
          first_quartile
          address_complements
          is_mad
          is_mad_updated_at
          mad_als_date
          reservation_agreement_ref
          designation_date_end
          dpe_ges
          dpe_conso
          entities_siret
          address
          postal_code
          country
          target_objective_ids
          auto_destroy_next_day
          last_update_by_lessor
          entity_rex_ids
          commercialized
          is_public
          last_update_by_lessor
          accept_low_occupation
          active_reservations
          delai_reponse
          picture_urls
          reference_mad
          lessor_orientations
          reservation_holder_id
          reserving_siren
          reserving_business_name
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        MUTATE_PARAMS = %i[
          lessor_code
          rent_with_charges
          lessor_siren
          requalification
          dalo
          entry_date
          external_ref
          communicate_exact_address
          typology
          lessor_status
          type
          age
          district
          publication_end_date
          expiry_days
          rent_amount
          surface
          rooms
          exclusivity_end_date
          nb_of_eligible_employees
          nb_of_requests_from_employees
          availability_date
          description
          details
          effort_rate
          min_effort_treshold
          priority_public
          mobility
          application_following_contact
          bedrooms
          bathrooms
          heating_type
          hot_water_distribution
          floor
          floors
          elevators
          has_gardian
          outside
          cellars
          parking_charges
          parking_type
          address_complements
          pmr
          kitchen_type
          mutation
          threshold_type
          threshold_subtype
          individual_heating
          date_publication_start
          date_publication_end
          lessor_id
          kind
          building_type
          allready_rent
          guarantee_deposit
          rental_charges
          offer_status
          min_effort_rate
          already_rent
          tense_area
          subsidiary_company
          rpls_id
          operation_name
          administrator
          rent_with_charges
          administration_area
          potential_request_name_town
          potential_request_name_county
          number_of_companies_30_minutes_around
          construction_type
          construction_year
          address_can_be_shared
          qpv
          total_floors
          address_extended
          test_created_at
          already_inhabited
          new_offer
          als_siren
          postal_code
          insee_code
          residence_code
          residence_title
          agency_id
          agency_name
          lessor_contact_name
          lessor_email
          lessor_phone_number
          gardens
          green_area
          condo
          furnished_lodging
          virtual_visit
          duplex
          first_quartile
          other_quartiles
          lessor_convention_reference
          disability_accesses
          terraces_or_balconies
          first_quartile
          mad_als_date
          reservation_agreement_ref
          dpe_ges
          dpe_conso
          ordered_picture_ids
          entities_siret
          commercialized
          is_public
          single_parent
          address
          postal_code
          country
          latitude
          longitude
          auto_destroy_next_day
          is_from_form
          accept_low_occupation
          delai_reponse
          picture_urls
          reference_mad
          lessor_orientations
          reserving_siren
        ].freeze

        LESSOR_FORBIDDEN_PARAMS = %i[
          als_siren
          dalo
          status
          status_updated_at
        ]

        def self.acceptable_params
          ACCEPTABLE_PARAMS
        end 

        def self.regex_params
          REGEX_PARAMS
        end 

        def mutate_params
          current_user && current_lessor ? MUTATE_PARAMS - LESSOR_FORBIDDEN_PARAMS : MUTATE_PARAMS + LESSOR_FORBIDDEN_PARAMS
        end 

        def handle_offers_eligibility
          applicant = Hermes::Applicant.find_by(id: @user&.main_applicant_id)
          hr = applicant&.first_active_request
          @query_precision ||= {}

          eligibility_query = if %w[Administrator bailleur].include? current_user_roles
            {}
          else
            if hr&.all_eligibility_query
              hr.all_eligibility_query.merge({published: true})
            else           
              {is_public: true}
            end
          end

          @query_precision = { :$and =>[@query_precision,eligibility_query]}
        end

        def include_relationships
          [:pictures]
        end

        def search
          index
        end

        def index_parameters_settings
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS
        end

        def create
          sanitize_string_as_integer
          # return render_error_not_mutable(not_mutable) unless not_mutable.empty?
          @model.current_user_id = @user.id
          user_id = @user.id
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          Rails.logger.debug "TEST before #{user_id}"
          user_id = param_user if @user.has_role?('Administrator') && param_user
          Rails.logger.debug "TEST #{mutate_valid_params} after #{user_id}"
          sanitize_status
          @instance = @model.new(mutate_valid_params.merge(user_id: user_id))

          lessor_criteria = @instance.lessor_code == "" ? { "$in": [nil, ""]} : @instance.lessor_code 
          found_instance = Hermes::HousingOffer.order_by(created_at: :desc).find_by(
            external_ref: @instance.external_ref, 
            lessor_siren: @instance.lessor_siren,
            lessor_code: lessor_criteria,
          )

          if !found_instance || (found_instance && found_instance.need_to_duplicate_offer)
 
            @instance.with_lock(reload: true) do
              @instance.last_update_by_lessor = DateTime.current
              return respond_with_errors(@instance) unless @instance.save
              render(jsonapi: @instance, serializer: @serializer, status: :created)
            end
          else
         
            @instance = found_instance
            @instance.with_lock(reload: true) do
              return render_json_error(:unprocessable_entity, :not_writers) unless @instance.reload && @instance.writable_by?(@user)
              valid_params = mutate_valid_params
              valid_params[:user_id] = params['data']['attributes'][:user_id] if params['data']['attributes'][:user_id] && @user.has_role?('Administrator')
              valid_params.delete(:lessor_status)
              valid_params.delete(:offer_status)
              valid_params.delete(:reserving_business_name)
              valid_params.delete(:details) unless found_instance.details.blank?

              return render_json_error(:parameters_not_present, :no_valid_parameters) unless valid_params.present?
              Rails.logger.debug "TEST valid_params are #{valid_params}"
              @instance.last_update_by_lessor = DateTime.current
              return respond_with_errors(@instance) unless @instance.update(valid_params) && @instance.save

              render(jsonapi: @instance, serializer: @serializer)
            end

          end
        end

        def aggregation_with_eligibility(other_query = nil)
          aggregation = if ["Administrator", "bailleur", "entreprise", "superadministrateur", "cre", "rdsl", "collaborateur offre logement"].any? {|role| current_user_roles.include?(role)}
            index_aggregation
          else
            handle_offers_eligibility
            @query_precision = { :$or =>[@query_precision, other_query]} if other_query
            index_aggregation(no_acl: true)
          end
        end 

        def index
          index_parameters_settings
          aggregation = aggregation_with_eligibility
          applicant = Hermes::Applicant.find_by(id: @user&.main_applicant_id)

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          if params[:postal_codes] && params[:postal_codes].is_a?(Array)
            aggregation = aggregation.search_by_postal_codes(params[:postal_codes])
          end

          if params[:longitude] && params[:latitude]
            distance = params[:distance] || 20
            unit = params[:unit] || :km
            ids = @model.near([params[:longitude], params[:latitude]], distance, unit: unit).map { |ho_v| ho_v['_id'] }
            aggregation = aggregation.where(id: { :$in => ids })
          end

          if params[:entity_ids].present? # params[:entity_ids] { "entity_ids": { "$in": [] } }
            offer_ids = Hermes::HousingEntityOffer.where(:entity_id => params[:entity_ids]).only([:housing_offer_id]).pluck(:housing_offer_id)
            aggregation = aggregation.where(:id.in => offer_ids)
          end

          @options = { eligible_requests_nb: params['options']&.include?('eligible_requests_nb')}

          paginate(
            aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def show
          @options = {
            include: %i[user hermes-lessor entities threshold],
            fields: {'hermes-lessor'.to_sym => ['title']},
            eligible_count: params['options']&.include?('eligible_count'),
            eligible_requests_nb: params['options']&.include?('eligible_requests_nb')
          }
          base_query = if %w[Administrator bailleur entreprise superadministrateur cre rdsl].any? {|role| current_user_roles.include?(role)}
                         @model.readable_by(@user)
                       elsif is_collaborator_offer
                         @model
                       else
                         handle_offers_eligibility if @user
                         @model
                       end
          @instance = base_query.find_by(id: params[:id], **@query_precision) if params[:id]

          super
        end

        def update
          @model.current_user_id = @user.id
          sanitize_status
          sanitize_string_as_integer
          # return render_error_not_mutable(not_mutable) unless not_mutable.empty?

          return render_json_error(:not_found, :object_not_found) unless @instance
          @instance.with_lock(reload: true) do
            super
          end

        end

        def is_collaborator_offer
          current_user.auth_roles&.include?('collaborateur offre logement')
        end

        def cotations_order_params
          o_params = order_params.to_h
          if o_params["cotations"] && @instance&.insee_code
            o_params["cotations.#{@instance.insee_code}"] = o_params['cotations']
            o_params.delete("cotations")
          end 
          o_params
        end 

        def all_eligible_requests
          return Hermes::HousingRequest.where(id: "") unless @instance.eligibility_query_base
          
          Hermes::HousingRequest.where(@instance.all_eligibility_query).where(build_query(Hermes::HousingRequest)).order_by(cotations_order_params)
        end

        def eligible_requests
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_action, :not_manager) unless can_manage_offers? || @user.auth_roles&.include?('collaborateur offre logement')

          acceptable_params *Hermes::Api::Dmo::HousingRequestsController.acceptable_params
          regex_params *Hermes::Api::Dmo::HousingRequestsController.regex_params
          valid_operators *VALID_OPERATORS

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          aggregation = @instance.eligibles_requests(params[:eligibility_type] || 'department' ).where(build_query(Hermes::HousingRequest)).order_by(cotations_order_params)
          
          entities = nil

          if params[:reserving_siren].present?
            entities = Hermes::ReservationHolder.find_entities_from_siren(params[:reserving_siren])
            aggregation = aggregation.where(:entity_id.in => entities) unless entities.nil?
          end 

          other_count = {}
          Hermes::HousingOffer.eligibility_types.each do |type|
            if params['options']&.include?("#{type}_count")
              eligibles_requests =  @instance.eligibles_requests(type).where(build_query(Hermes::HousingRequest))
              eligibles_requests = eligibles_requests.where(:entity_id.in => entities) unless entities.nil?
              other_count["#{type}_count"] = eligibles_requests.count
            end 
          end 

          options[:other_count] = other_count unless other_count.empty?

          paginate_with_other_count(
            aggregation,
            each_serializer: Hermes::HousingRequestSerializer,
            **options
          )
        end

        def is_eligible_request
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:not_acceptable, :missing_als_id) if params[:als_id].blank?
          return render_json_error(:not_found, :request_not_found) unless hr = Hermes::HousingRequest.find_by(als_id: params[:als_id])
      
          render json: { is_eligible: !!Hermes::HousingRequest.where(@instance.all_eligibility_query).find_by(id: hr.id.to_s) }
        end 

        def eligible_requests_rights_filtered
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unauthorized, :not_manager) unless can_manage_offers? || @user.auth_roles&.include?('collaborateur offre logement')

          acceptable_params *Hermes::Api::Dmo::HousingRequestsController.acceptable_params
          regex_params *Hermes::Api::Dmo::HousingRequestsController.regex_params
          valid_operators *VALID_OPERATORS

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          entity_ids = current_user_entity_ids
          entity_ids = current_user_entity_ids & params[:entity_ids] if params[:entity_ids].present?

          paginate(
            all_eligible_requests.where(:entity_id.in => entity_ids),
            each_serializer: Hermes::HousingRequestSerializer,
            **options
          )
        end 


        def entity_statuses
          @instance = @model.find_by(id: params[:id]) if params[:id]
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:not_acceptable, :missing_entity_id_params) unless params[:entity_id]

          can_read_entity = Hermes::Entity.readable_by(current_user).find_by(id: params[:entity_id])
          return render_json_error(:unauthorized, :no_access_to_entity) unless can_read_entity

          aggregation = Hermes::Status.where(housing_offer_id: @instance.id, status_label: 'entity_status', performer_id: params[:entity_id])

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}
          paginate(
            aggregation,
            each_serializer: Hermes::StatusSerializer,
            **options
          )
        end

        def all_applications
          return render_json_error(:not_found, :instance_not_found) unless @instance && can_manage_offers?
          # return render_json_error(:unprocessable_action, :not_manager) unless can_manage_offers?
          acceptable_params *Hermes::Api::Dmo::HousingApplicationsController.acceptable_params
          regex_params *Hermes::Api::Dmo::HousingApplicationsController.regex_params
          valid_operators *VALID_OPERATORS

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}
          aggregation = Hermes::HousingApplication.actives.where(housing_offer_id: @instance.id).where(build_query(Hermes::HousingApplication)).order(cotations_order_params)

          paginate(
            aggregation,
            each_serializer: Hermes::HousingApplicationLightSerializer,
            **options
          )
        end

        def all_applications_rights_filtered
          return render_json_error(:not_found, :instance_not_found) unless @instance && can_manage_offers?

          acceptable_params *Hermes::Api::Dmo::HousingApplicationsController.acceptable_params
          regex_params *Hermes::Api::Dmo::HousingApplicationsController.regex_params
          valid_operators *VALID_OPERATORS

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          entity_ids = current_user_entity_ids
          entity_ids = current_user_entity_ids & params[:entity_ids] if params[:entity_ids].present?

          aggregation = @instance.find_housing_application_with_entity_id(entity_ids).where(build_query(Hermes::HousingApplication)).order(cotations_order_params)

          paginate(
            aggregation,
            each_serializer: Hermes::HousingApplicationLightSerializer,
            **options
          )
        end

        def paginate_applications(applications)
          acceptable_params *Hermes::Api::Dmo::HousingApplicationsController.acceptable_params
          regex_params *Hermes::Api::Dmo::HousingApplicationsController.regex_params
          valid_operators *VALID_OPERATORS
          applications = applications.readable_by(@user).where(build_query(Hermes::HousingApplication))
                           .order(order_params)
                           .includes(housing_request: [:main_applicant])
          options = {}
          options[:join_housing_request] = true
          paginate(
            applications,
            each_serializer: HousingApplicationSerializer,
            **options
          )
        end

        def selected_applications
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_action, :not_manager) unless can_manage_offers? #todo : a vérifier lorsqu'on aura la gestion de droits spécifiée

          applications = Hermes::HousingApplication.selected_by_offer @instance.id
          paginate_applications(applications)
        end

        def can_manage_offers?
          (MANAGEMENT_ROLES & current_user_roles)&.length&.send('>', 0)
        end

        def render_json_error_not_active_selected(housing_application_ids, concerned_applications, housing_offer_id)
          errors = []
          diff = housing_application_ids.uniq - concerned_applications.pluck(:id).map(&:to_s)
          diff.each do |id|
            ha = Hermes::HousingApplication.find(id)
            if !ha
              errors.push("Candidature (#{id}) introuvable") 
            elsif !ha.is_active
              errors.push("Candidature (id : #{id} / als_id : #{ha.housing_request.als_id}) inactive, statut : #{ha.status}, nud : #{ha.housing_request.unique_numbers.join(',')}") 
            else
              errors.push("Candidature (id : #{id} / als_id : #{ha.housing_request.als_id}) non associée à cette offre (#{housing_offer_id}), statut : #{ha.status}, nud : #{ha.housing_request.unique_numbers.join(',')}") 
            end 
          end 

          render(json: errors, status: :unprocessable_entity)
        end 

        def post_selected_applications
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_entity, :not_manager) unless can_manage_offers? #todo : a vérifier lorsqu'on aura la gestion de droits spécifiée

          housing_application_ids = params.dig(:data, :attributes, :housing_application_ids)
          return render_json_error(:unprocessable_entity, :no_valid_parameters) unless housing_application_ids

          concerned_applications = Hermes::HousingApplication.where(
            is_active: true,
            :id.in => housing_application_ids, 
            housing_offer_id: @instance.id
          )

          unless housing_application_ids.uniq.length == concerned_applications.count
            return render_json_error_not_active_selected(housing_application_ids, concerned_applications, @instance.id) 
          end 

          @instance.update(
            lessor_status: "EN ATTRIBUTION",
            offer_status: "EN COURS D'ATTRIBUTION"
          )

          concerned_applications.each do |ha| 
            ha.update(is_active: true, status: "TRANSMIS BAILLEUR")
          end

          Hermes::HousingApplication.where(
            is_active: true,
            housing_offer_id: @instance.id,
            :id.nin => housing_application_ids,
          ).each do |ha| 
            ha.update(is_active: false, status: "NON RETENUE")
          end

          options = {join_housing_request: true}
          paginate(
            concerned_applications,
            each_serializer: HousingApplicationSerializer,
            join_housing_request: true
          )
        end

        def from_param
          from = params['from']
          return render_json_error(:unprocessable_entity, :missing_from_parameter) unless from
          begin
            DateTime.parse(from)
          rescue
            return render_json_error(:unprocessable_entity, :malformed_from_parameter)
          end
        end

        def from_to_query
          from = from_param
          return from unless from.is_a?(DateTime)

          query = {status_updated_at: {:$gte => from}}
          to = DateTime.parse(params[:to]) rescue nil
          query[:status_updated_at][:$lte] = to if to.is_a? DateTime
          query
        end

        def with_validated_applications
          from_to = from_to_query
          return from_to unless from_to.is_a?(Hash)

          application_query = {
            status: VALIDATED_APPLICATION_STATUS,
            is_active: true
          }.merge(from_to)

          validated_applications = Hermes::HousingApplication.where(
            application_query
          )
          housing_offer_ids = validated_applications.pluck(:housing_offer_id)
                                .uniq
                                .compact
          @query_precision = {:id.in => housing_offer_ids}
          index_parameters_settings
          aggregation = index_aggregation
          options = {}

          unless ['false', false].include?(params[:pagination])
            paginate(
              aggregation,
              each_serializer: Hermes::HousingOfferSerializer,
              **options
            )
          else
            render(
              jsonapi: aggregation,
              each_serializer: Hermes::HousingOfferSerializer,
              **options
            )
          end
        end

        def with_validated_applications_for
          def include_relationships
            []
          end

          from_to = from_to_query
          return from_to unless from_to.is_a?(Hash)
          return render_json_error(:not_found, :object_not_found) unless @instance

          acceptable_params *Hermes::Api::Dmo::HousingApplicationsController.acceptable_params
          regex_params *Hermes::Api::Dmo::HousingApplicationsController.regex_params
          valid_operators *VALID_OPERATORS

          applications_query = {
            housing_offer_id: @instance.id,
            status: VALIDATED_APPLICATION_STATUS,
            is_active: true
          }.merge(from_to)
          @query_precision = applications_query
          aggregation = index_aggregation(target_model: Hermes::HousingApplication)
          options = {embedded_applicant: true}

          unless ['false', false].include?(params[:pagination])
            paginate(
              aggregation,
              each_serializer: Hermes::HousingApplicationSerializer,
              **options
            )
          else
            render(
              jsonapi: aggregation,
              each_serializer: Hermes::HousingApplicationSerializer,
              **options
            )
          end
        end

        def paginate_history(history)
          acceptable_params *%i[lessor_objective_id]
          history = history.where(build_query(Hermes::LessorObjectiveHistory)).order(order_params)
          paginate(
            history,
            each_serializer: Hermes::LessorObjectiveHistorySerializer
          )
        end

        def history
          if current_user_roles.include?('bailleur')
            @instance = @model.readable_by(@user).find_by(id: params[:id], **@query_precision) if params[:id]
          end

          return render_json_error(:not_found, :instance_not_found) unless @instance

          paginate_history(@instance.history)
        end

        def reference_exist
          return render_json_error(:unprocessable_entity, :missing_external_ref_parameter) unless params[:external_ref]

          offers = Hermes::HousingOffer.where({
            offer_status: { '$nin': Hermes::HousingOffer.end_of_life_statuses},
            external_ref: params[:external_ref]
          })

          paginate(
            offers,
            each_serializer: @serializer
          )
        end

        def add_housing_channel
          @instance ||= @model.find_by(id: params[:id])
          keys = params[:keys] || [params[:key]]
          # If an array of keys is given, delete all existing keys to remplace by array
          if @instance && params[:keys].present?
            @instance.set(housing_channel_keys: [], updated_at: DateTime.current)
          end
          update_housing_channel do
            keys.each do |key|
              if Hermes::HousingChannel.find_by(key: key)
                @instance.update_housing_channel_keys!(key)
              else
                render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
              end
            end
          end
        end

        def remove_housing_channel
          keys = params[:keys] || [params[:key]]
          update_housing_channel do
            keys.each do |key|
              if Hermes::HousingChannel.find_by(key: key)
                @instance.update_housing_channel_keys!(key, false)
              else
                render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
              end
            end
          end
        end

        def with_recount_eligible_requests_nb
          return render_json_error(:unprocessable_entity, :housing_offer_ids_params_is_missing) unless params[:housing_offer_ids].present?
          entities = nil
      
          if params[:reserving_siren].present?
            entities = Hermes::ReservationHolder.find_entities_from_siren(params[:reserving_siren])
          end 
          
          @options = {
            eligible_requests_nb: true,
            entities_used_for_eligible_requests_nb: entities,
          }

          housing_offer_user_application_ids = Hermes::HousingApplication.where(user: current_user).pluck(:housing_offer_id)
          aggregation = aggregation_with_eligibility({id: {"$in": housing_offer_user_application_ids}}).where(:id.in => params[:housing_offer_ids])

          paginate(
            aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def with_recount_eligible_requests_nb_rights_filtered
          return render_json_error(:unprocessable_entity, :housing_offer_ids_params_is_missing) unless params[:housing_offer_ids].present?
      
          entities = Hermes::UserRight.where(user: current_user).pluck(:entity_id).map(&:to_s) - [nil]
          entities = entities & params[:entity_ids] if params[:entity_ids].present?
          
          @options = {
            eligible_requests_nb: true,
            entities_used_for_eligible_requests_nb: entities,
            recount_applicated_nb: true,
          }

          paginate(
            Hermes::HousingOffer.where(:id.in => params[:housing_offer_ids]),
            each_serializer: @serializer,
            **@options
          )
        end 

        def resend_flux5_all_missing_offers
          date = params[:date].present? ? Date.parse(params[:date]) : Date.today - 7.days
          offers = Hermes::HousingOffer.where(is_mad: true, last_successful_xml_sent_date: nil, :offer_status.nin => Hermes::HousingOffer.end_of_life_statuses, :created_at.gte => date.beginning_of_day)
          offer_ids = offers.pluck(:id).map(&:to_s)
          offer_ids.each do |offer_id|
            ExecuteFlux5TaskJob.perform_later(offer_id)
          end 

          render(json: { success: true, offer_ids: offer_ids }, status: '200') 
        end 

        def resend_flux5
          return render_json_error(:not_found, :instance_not_found) unless @instance

          res = ExecuteFlux5TaskJob.perform_now(@instance.id.to_s)

          return render_json_error(:unprocessable_entity, :error_on_flux5) if !res || res.code != 200

          render(json: { success: true }, status: '200') 
        end 
      

        def simulate_eligible_requests_count
          ap params
          if [
            :threshold_type,
            :threshold_subtype,
            :rent_with_charges,
            :insee_code,
            :typology,
          ].any? { |p| !params[p].present?}
            return render_json_error(:unprocessable_entity, :missing_params)
          end 

          offer = Hermes::HousingOffer.new(
            offer_status: 'LOGEMENT PUBLIE',
            threshold_type: params[:threshold_type],
            threshold_subtype: params[:threshold_subtype],
            rent_with_charges: params[:rent_with_charges].to_f,
            insee_code: params[:insee_code],
            typology: params[:typology],
          )

          offer.update_threshold
          offer.threshold.set_offer_thresholds(offer)
          offer.update_min_revenues
          offer.update_department
          offer.update_housing_channel_keys

          offer.update_eligibility_query
          query = offer.eligibility_query('seeked_bordering')

          [
            :single_parent,
            :priority_public,
            :mutation,
          ].each do |att|
            query.delete(att)
          end 

          complete_eligible_requests_count = Hermes::HousingRequest.where(query.merge(is_active: true, request_status: 'COMPLETE')).count
          possible_eligible_requests_count = Hermes::HousingRequest.where(query.merge(is_active: true, :request_status.nin => ['ANNULEE', 'SATISFAITE'])).count

          render(json: { complete_eligible_requests_count: complete_eligible_requests_count, possible_eligible_requests_count: possible_eligible_requests_count }, status: '200') 
        end 

        private

        def update_housing_channel
          @instance ||= @model.find_by(id: params[:id])
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unauthorized, :not_allowed) unless can_manage_housing_channel?

          if (params[:keys].present? && params[:keys].is_a?(Array)) || params[:key].present?
            yield if block_given?

            render json: {
              success: true,
              channels: @instance.housing_channel_keys
            }, status: :ok
          else
            raise ActionController::ParameterMissing.new('key or keys')
          end

        rescue ActionController::ParameterMissing => e
          render_json_error(:not_acceptable, :parameter_missing, {message: e.message})
        rescue Hermes::Errors::HousingChannelError => e
          render_json_error(:unprocessable_entity, :housing_channel_error, {message: e.message})
        end

        def can_manage_housing_channel?
          if current_user_roles.include?('bailleur')
            lessor = Hermes::Lessor.find_by(id: @instance.lessor_id)
            lessor.present? && lessor.readable_by?(@user)
          else
            current_user_roles.include?('Administrator')
          end
        end

        def sanitize_status
          # only offer.lessor can setup Hermes::HousingOffer::ATTRIBUTION_STATUSES
          if Hermes::HousingOffer::ATTRIBUTION_STATUSES.include?(params[:data][:attributes][:offer_status])
            if current_lessor.nil? || current_lessor&.id&.to_s != mutate_valid_params[:lessor_id]
              @instance = @model.find(params[:id] || 0)
              params[:data][:attributes][:offer_status] = @instance&.offer_status_was
            end
          end
        end

        def sanitize_string_as_integer
          %w[elevator garden cellar disability_access].each do |extra_name|
            next unless params[:data][:attributes].key?(extra_name.pluralize.to_sym)

            val = params[:data][:attributes][extra_name.pluralize.to_sym] 
            if val.nil? || val.class == String
              params[:data][:attributes][extra_name.pluralize.to_sym] = ['1', 'O'].include?(val) ? 1 : val.to_i
            end                    
          end
        end 

        def log_skipable?
          # skip after_filter declared in core
          %i[index].include? action_name.to_sym
        end

        private 

        def current_user_entity_ids
          Hermes::UserRight.where(user: current_user).pluck(:entity_id).map(&:to_s) - [nil]
        end 
      end
    end
  end
end
