# frozen_string_literal: true

module Hermes
  # HousingOffer serializer
  class HousingOfferSerializer < Hermes::ApplicationSerializer
    attribute :is_from_form
    attribute :requalification
    attribute :dalo
    attribute :als_id
    attribute :entry_date
    attribute :external_ref
    attribute :communicate_exact_address
    attribute :typology
    attribute :type
    attribute :age
    attribute :district
    attribute :department
    attribute :pictures
    attribute :pictures_nb
    attribute :ordered_picture_ids
    attribute :main_picture
    attribute :publication_end_date
    attribute :expiry_days
    attribute :rent_amount
    attribute :rent_with_charges
    attribute :surface
    attribute :rooms
    attribute :exclusivity_end_date
    attribute :nb_of_eligible_employees
    attribute :nb_of_requests_from_employees
    attribute :availability_date
    attribute :description
    attribute :details
    attribute :effort_rate
    attribute :designation_date_end
    attribute :min_effort_treshold
    attribute :single_parent
    attribute :reserved
    attribute :already_rent
    attribute :reservation_agreement_ref
    attribute :priority_public
    attribute :mobility
    attribute :bedrooms
    attribute :bathrooms
    attribute :heating_type
    attribute :hot_water_distribution
    attribute :application_following_contact
    attribute :floor
    attribute :floors
    attribute :elevators
    attribute :has_elevator
    attribute :has_gardian
    attribute :outside
    attribute :cellars
    attribute :has_cellar
    attribute :parking_type
    attribute :pmr
    attribute :kitchen_type
    attribute :address
    attribute :postal_code
    attribute :country
    attribute :latitude
    attribute :longitude
    attribute :address_complements
    attribute :mutation
    attribute :threshold_type
    attribute :threshold_subtype
    attribute :individual_heating
    attribute :date_publication_start
    attribute :date_publication_end
    attribute :kind
    attribute :building_type
    attribute :guarantee_deposit
    attribute :rental_charges
    attribute :parking_charges
    attribute :offer_status
    attribute :offer_status_id
    attribute :offer_status_updated_at
    attribute :lessor_status
    attribute :lessor_status_id
    attribute :lessor_status_updated_at
    attribute :ceil_category_1
    attribute :ceil_category_2
    attribute :ceil_category_3
    attribute :ceil_category_4
    attribute :ceil_category_5
    attribute :ceil_category_6
    attribute :ceil_category_7
    attribute :ceil_category_8
    attribute :ceil_category_9
    attribute :ceil_category_10
    attribute :ceil_category_11
    attribute :ceil_category_12
    attribute :floor_category_1
    attribute :floor_category_2
    attribute :floor_category_3
    attribute :floor_category_4
    attribute :floor_category_5
    attribute :floor_category_6
    attribute :floor_category_7
    attribute :floor_category_8
    attribute :floor_category_9
    attribute :floor_category_10
    attribute :floor_category_11
    attribute :floor_category_12
    attribute :min_revenues
    attribute :total_charges
    attribute :min_effort_rate
    attribute :tense_area
    attribute :subsidiary_company
    attribute :rpls_id
    attribute :operation_name
    attribute :administrator
    attribute :administration_area
    attribute :potential_request_name_town
    attribute :potential_request_name_county
    attribute :number_of_companies_30_minutes_around
    attribute :construction_type
    attribute :construction_year
    attribute :address_can_be_shared
    attribute :qpv
    attribute :total_floors
    attribute :address_extended
    attribute :test_created_at
    attribute :already_inhabited
    attribute :new_offer
    attribute :lessor_name
    attribute :lessor_siren
    attribute :als_siren
    attribute :first_quartile
    attribute :postal_code
    attribute :insee_code
    attribute :residence_code
    attribute :residence_title
    attribute :agency_id
    attribute :agency_name
    attribute :lessor_contact_name
    attribute :lessor_email
    attribute :lessor_phone_number
    attribute :gardens
    attribute :has_garden
    attribute :green_area
    attribute :terraces_or_balconies
    attribute :dpe_ges
    attribute :dpe_conso
    attribute :condo
    attribute :furnished_lodging
    attribute :virtual_visit
    attribute :duplex
    attribute :first_quartile
    attribute :other_quartiles
    attribute :lessor_convention_reference
    attribute :disability_accesses
    attribute :has_disability_access
    attribute :lessor_id
    attribute :lessor
    attribute :entities_siret
    attribute :entity_ids
    attribute :threshold_id
    attribute :agency_id
    attribute :applicated_nb
    attribute :reference
    attribute :release_date
    attribute :commercialized
    attribute :is_public
    attribute :is_mad
    attribute :is_mad_updated_at
    attribute :mad_als_date
    attribute :target_objective_ids
    attribute :auto_destroy_next_day
    attribute :last_update_by_lessor
    attribute :housing_channel_keys
    attribute :accept_low_occupation
    attribute :active_reservations
    attribute :active_reservations_rex_ids
    attribute :last_send_has_error
    attribute :delai_reponse
    attribute :reserving_siren
    attribute :reservation_holder_id
    attribute :reserving_business_name

    attribute :hash_xml
    attribute :last_successful_xml_sent_date
    attribute :lessor_code

    attribute :reference_mad
    attribute :lessor_orientations
    attribute :picture_urls

    attribute :eligible_count, if: :eligible_count?
    attribute :eligible_requests_nb, if: :eligible_requests_nb?
    attribute :possible_eligible_requests_nb, if: :eligible_requests_nb?

    attribute :eligibility_query_base
    attribute :eligibility_query_department
    attribute :eligibility_query_seeked
    attribute :eligibility_query_bordering
    attribute :eligibility_query_seeked_bordering
    attribute :eligibility_query_other_from_department
    
    if HERMES_CONFIG['DEBUG_ERRORS'] == false
      attribute :last_xml_validation_report
    end
    
    type 'housing_offers'

    def added_to_object_cache_key
      if eligible_requests_nb? || 
        eligible_count?
        return (DateTime.current.to_i / 10).to_s
      end
      "" 
    end 

    def applicated_nb
      return object.applicated_nb unless @instance_options[:recount_applicated_nb]
      object.find_housing_application_with_entity_id(@instance_options[:entities_used_for_eligible_requests_nb]).count
    end 
    
    def lessor_id
      object.lessor_id&.to_s
    end
    
    def entity_ids
      object.entity_ids&.map(&:to_s)
    end
    
    def target_objective_ids
      object.target_objective_ids&.map(&:to_s)
    end
    
    def agency_id
      object.agency_id&.to_s
    end
    
    # def agency_name
    #   object.agency&.title
    # end
    
    def threshold_id
      object.threshold_id&.to_s
    end
    
    def offer_status_id
      object.offer_status_id.to_s
    end
    
    def lessor_status_id
      object.lessor_status_id.to_s
    end
    
    def lessor
      if object.lessor
        Hermes::LessorSerializer.new(
          object.lessor, scope:
          scope, root: false,
          reduced: true)
      end
    end
    
    # pictures
    def ordered_picture_ids
      object.ordered_picture_ids&.map(&:to_s) || []
    end
    
    def default_picture
      {
        "is_default" => true,
        "thumb240" => google_map_link(240),
        "thumb1000" => google_map_link(1000),
        "full_size" => google_map_link(1500),
        "housing_offer_id" => object.id
      }
    end
    
    def google_map_link(size)
      require 'cgi'
      key = (HERMES_CONFIG['google_street_view'] && HERMES_CONFIG['google_street_view']['key']) || "AIzaSyB44UtIXYeMCKhtwUWeUdO16KVDMUAHHqQ"
      signature = (HERMES_CONFIG['google_street_view'] && HERMES_CONFIG['google_street_view']['signature']) || "svUpEVUsO9VuYG24KbDi89Gndw0="
      url = "https://maps.googleapis.com/maps/api/streetview?size=" + size.to_s + "x" + size.to_s +
        "&location=" + CGI.escape(object.address.to_s + " " + object.postal_code.to_s + " " + object.city.to_s) +
        "&key=" + key
      GoogleUrlSigner.sign(url, signature)
    end
    
    def main_picture
      return default_picture unless main = object.main_picture
      main.is_a?(Hermes::Picture) ? Hermes::PictureSerializer.new(main, scope: scope, root: false) : format_picture_url(main)
    end
    
    def format_picture_url(picture_url)
      {
        "is_picture_url"=> true,
        "thumb240"=> picture_url["thumb240"]||picture_url["thumb1000"]||picture_url["full_size"],
        "thumb1000"=> picture_url["thumb1000"]||picture_url["full_size"]||picture_url["thumb240"],
        "full_size"=> picture_url["full_size"]||picture_url["thumb1000"]||picture_url["thumb240"],
        "housing_offer_id"=> object.id
      }
    end
    
    def pictures
      if object.ordered_pictures && !object.ordered_pictures.empty?
        object.ordered_pictures.map do |picture|
          next unless picture
          Hermes::PictureSerializer.new(picture, scope: scope, root: false)
        end
      elsif object.picture_urls && !object.picture_urls.empty?
        object.picture_urls.map do |picture|
          next unless picture
         format_picture_url(picture)
        end
      else
        return [default_picture]
      end
    end
    
    def pictures_nb
      ordered_picture_ids.length
    end

    def eligible_count?
      @instance_options[:eligible_count] == true
    end

    def eligible_count
      eligible_count = {}
      Hermes::HousingOffer.eligibility_types.each do |type|
        eligible_count["#{type}_count"] =  object.eligibles_requests(type).count
      end
      eligible_count["total_count"] = eligible_count["seeked_bordering_count"] + eligible_count["other_from_department_count"]
      eligible_count
    end 

    def eligible_requests_nb?
      @instance_options[:eligible_requests_nb] == true
    end

    def eligible_requests_nb
      requests = Hermes::HousingRequest.where(object.all_eligibility_query)
      return requests&.count unless @instance_options[:entities_used_for_eligible_requests_nb]
      requests&.where(:entity_id.in => @instance_options[:entities_used_for_eligible_requests_nb])&.count
    end 

    def possible_eligible_requests_nb
      query = object.all_eligibility_query
      query[:request_status] = {'$ne': 'SATISFAITE' }
      requests = Hermes::HousingRequest.where(query)
      return requests&.count unless @instance_options[:entities_used_for_eligible_requests_nb]
      requests&.where(:entity_id.in => @instance_options[:entities_used_for_eligible_requests_nb])&.count
    end 
    
  end
end
