# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'

dir_path = File.dirname(__FILE__)
schema_path = [dir_path, 'complementDemandeImhoweb.xsd'].join('/')
require [Hermes::Engine.root, %w[lib hermes bo_client tasks]].flatten.join('/').to_s

RSpec.describe Hermes::BoClient::SupplementRequestTask, type: :helper do
  let!(:xsd) { Nokogiri::XML::Schema(File.read(schema_path)) }

  it 'should load a valid XSD' do
    expect(xsd).to be_present
    expect(xsd.class).to be Nokogiri::XML::Schema
  end

  context 'with simple housing_request' do
    let!(:app) { create(:applicant)}
    let!(:hr) { create(:housing_request, is_active: true, main_applicant: app) }
    let!(:nud) {create(:nud, housing_request: hr, unique_number: '123456789012345678')}
    let!(:offer) { create(:housing_offer, insee_code: '34000') }

    let!(:task) { Hermes::BoClient::SupplementRequestTask.new(housing_request_id: hr.id)}
    let!(:task2) { Hermes::BoClient::SupplementRequestTask.new(housing_request_id: hr.id, insee_code: offer.insee_code)}

    it 'for housing_request' do
      task.validate_xml
      parsed_xml = hr.parse_xml(task.to_xml)
      expect(xsd.validate(Nokogiri::XML(task.to_xml)).length).to eq(0)
      expect(parsed_xml['complement_demande_imhoweb']['numUnique']).to eq(hr.unique_numbers.last)
    end

    it 'for housing_application' do
      task2.validate_xml
      parsed_xml = hr.parse_xml(task2.to_xml)
      expect(xsd.validate(Nokogiri::XML(task2.to_xml)).length).to eq(0)
      expect(parsed_xml['complement_demande_imhoweb']['numUnique']).to eq(hr.unique_numbers.last)
      expect(parsed_xml['complement_demande_imhoweb']['codeInseeInstruction']).to eq(offer.insee_code)

    end 
  end
end
